#!/usr/bin/env perl
#!/bin/env perl

## Name: fusionintron.pl
## Version: 1.2.6
## Date: May 17, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret

## refine introns into non-overlapped introns by getting the highest occurrence
# --input, input bed file = intron output from joinpairbed12.pl
# --output, output bed file
# --mincov, minimum number of reads containing an intron

use strict;
use warnings;
use Getopt::Long;

my $help;
my $input;
my $output = "output.bed";
my $mincov = 1;
my $line;
my @args;
my @args_old;
my @args_prev;
my $i;
my $count_old;
my $count_prev;
my @stock;
my @new_stock;
my @max_intron;
my @kept_introns;
my @current_intron;
my $max_count;

usage() if (@ARGV < 2 or
	    ! GetOptions("help|?" => \$help,
			 "input=s" => \$input,
			 "mincov=i" => \$mincov,
			 "output=s" => \$output)
	    or defined $help);

open(IN, $input) or die("Input file not found\n");
open(OUT, ">$output");

$line = <IN>;
if (!$line) {
    print "Empty input file\n";
    close(OUT);
    exit 0;
}
chomp($line);
@args_old = split("\t", $line);

while ($line = <IN>) {
    chomp($line);
    @args = split("\t", $line);
    $count_old = 1;
    @stock = ();
    push(@stock, $args_old[0], $args_old[1], $args_old[2], $count_old);
    
    #< while overlapped do taking the highest occurrence
    while ($line && $args[1] < $args_old[2] && $args[0] eq $args_old[0]) {
	if ($args[1]==$args_old[1] && $args[2]==$args_old[2]) {
	    $count_old++;
	    $stock[$#stock] = $count_old;
	    $line = <IN>;
	    if ($line) {
		chomp($line);
		@args = split("\t", $line);
	    }
	}
	else {
	    $count_prev = 0;
	    @args_prev = @args;
	    while ($line && $args[1]==$args_prev[1] && $args[2]==$args_prev[2] && $args[0] eq $args_prev[0]) {
		$count_prev++;
		$line = <IN>;
		if ($line) {
		    chomp($line);
		    @args = split("\t", $line);
		}
	    }
	    
	    if ($count_old < $count_prev) {
		@args_old = @args_prev;
		$count_old = $count_prev;
	    }
	    else {
		push(@stock, $args_prev[0], $args_prev[1], $args_prev[2], $count_prev);
	    }
	}
    }
    #>
    # print "@args_old\n";
    #print "fin: $line\n ";
    
    # print "@stock\n";
    #< take previously non-overlapped introns with the current highest-occurrence intron
    @new_stock = @stock;
    @kept_introns = ();
    @current_intron = (@args_old, $count_old);
    while ($#new_stock > 0) {
	@new_stock = ();
	$max_count = 0;
	@max_intron = ();
	#< take the highest-occurrence intron non-overlapped with the current intron
	for ($i=0; $i<$#stock; $i+=4) {
	    if ($stock[$i+2] < $current_intron[1] || $stock[$i+1] > $current_intron[2]) {
		push(@new_stock, $stock[$i], $stock[$i+1], $stock[$i+2], $stock[$i+3]);
		if ($stock[$i+3] > $max_count) {
		    $max_count = $stock[$i+3];
		    @max_intron = ($stock[$i], $stock[$i+1], $stock[$i+2], $stock[$i+3]);
		}
	    }                           
	}
	#>
	@current_intron = @max_intron;
	@kept_introns = (@current_intron, @kept_introns);
	@stock = @new_stock;
    }
    #>
    for ($i=0; $i<$#kept_introns; $i+=4) {
	if ($kept_introns[$i+3]>=$mincov) {
	    print OUT "$kept_introns[$i]\t$kept_introns[$i+1]\t$kept_introns[$i+2]\n";
	}
    }
    if ($count_old >=$mincov) {
	for ($i=0; $i<=$#args_old; $i++) {
	    print OUT "$args_old[$i]\t";
	}
	print OUT "\n";
    }
    @args_old = @args;
    if (eof && $line && $mincov <= 1) {
	for ($i=0; $i<=$#args_old; $i++) {
	    print OUT "$args_old[$i]\t";
	}
	print OUT "\n";
    }
}
close(IN);
close(OUT);
exit 0;

sub max {
    if ($_[0] > $_[1]) {
	$_[0];
    }
    else {
	$_[1];
    }
}

sub min {
    if ($_[0] < $_[1]) {
	$_[0];
    }
    else {
	$_[1];
    }
}

sub fusion {
    my ($a, $b) = @_;
    my @ref = @{$a};
    $ref[1] = max(${$a}[1], ${$b}[1]);	
    $ref[2] = min(${$a}[2], ${$b}[2]);

    return @ref;
}

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: perl fusionintron.pl --input BED [--output OUT] [--mincov MIN] [--help|?] \n";
    exit 1;
}
