#!/usr/bin/env perl
#!/bin/env perl

## Name: jointscexon.pl
## Version: 1.2.6
## Date: May 17, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret

# recreate exon list from gff file with exons in the attribute field
# --input, input gff file, containing transcripts and their exons in the attribute field
##e.g.
##transcript exons=
# --output, output gff with exons
# --exon, output exon list
# --transcript, output gff without exons

use strict;
use warnings;
use Getopt::Long;

my $help;
my $input;
my $output = "output.gff";
my $exon = "exons.tsv";
my $transcript = "transcripts.gff";

my $line;
my @args;
my @argstab;
my @argsexon;
my @argsID;
my $exonnum;

usage() if (@ARGV < 2 or
	    ! GetOptions("help|?" => \$help,
			 "input=s" => \$input,
			 "exon=s" => \$exon,
			 "transcript=s" => \$transcript,
			 "output=s" => \$output)
	    or defined $help);

if (defined $input) {
    open(STDIN, $input);
}
if (defined $output) {
    open(STDOUT, ">$output");
}
open(EXON, ">$exon");
open(TSC, ">$transcript");

while ($line=<STDIN>) {
    chomp($line);
    @args = split("exons=", $line);
    $args[0] = &trim($args[0]);
    print STDOUT ("$args[0]\n");
    print TSC ("$args[0]\n");
    @argstab = split("\t", $args[0]);
    if (!$args[1]) {
	$args[1] = "$argstab[3] $argstab[4]";
    }
    @argsexon = split(" ", $args[1]);
    @argsID = split("ID", $argstab[8]);
    @argsID = split(";", $argsID[1]);
    $argsID[0] = &trim($argsID[0]);
    
    @args = split(" ", $args[1]);
    print EXON (join("\t", @args));
    print EXON "\n";
    
    for (my $i=0; $i<$#argsexon; $i=$i+2) {
	$exonnum = $i/2 + 1;
	print STDOUT "$argstab[0]\t$argstab[1]\texon\t$argsexon[$i]\t$argsexon[$i+1]\t$argstab[5]\t$argstab[6]\t$argstab[7]\tParent=$argsID[0];Exon_num=$exonnum\n";
    }
}

close(STDIN);
close(STDOUT);
close(EXON);
close(TSC);
exit 0;

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: perl jointscexon.pl --input GFF [--exon EXON] [--transcript TSC] [--output OUT] [--help|?] \n";
    exit 1;
}

sub trim {
    my $str = shift;
    
    ##-- remove white spaces at the beginning and at the end
    $str =~ s/^\s+|\s+$//g;
    ##-- remove semicolons at the beginning and at the end
    $str =~ s/^\;+|\;+$//g;
    ##-- remove quotes at the beginning and at the end
    $str =~ s/^\"+|\"+$//g;
    ##-- remove ='s at the beginning and at the end
    $str =~ s/^\=+|\=+$//g;
    return $str;
}
